@@ -128,7 +128,7 @@ DEF_Explain
  * for the most part the only code that acts on 'em.  (Hmmm... mod_main.c?)
  */
 
-int standalone = 1;
+int standalone=1;
 uid_t user_id;
 char *user_name;
 gid_t group_id;
@@ -268,7 +268,7 @@ accept_mutex_init(pool *p)
     if (lock_fd == -1)
     {
 	perror ("open");
-	fprintf (stderr, "Cannot open lock file\n");
+	fprintf (stderr, "Cannot open lock file: %s\n", lock_fname);
 	exit (1);
     }
     unlink(lock_fname);
@@ -666,8 +666,27 @@ static void setup_shared_mem(void)
 
 #elif defined(MAP_ANON) || defined(MAP_FILE)
 /* BSD style */
+#ifdef CONVEXOS11
+    /*
+     * 9-Aug-97 - Jeff Venters (venters@convex.hp.com)
+     * ConvexOS maps address space as follows:
+     *   0x00000000 - 0x7fffffff : Kernel
+     *   0x80000000 - 0xffffffff : User
+     * Start mmapped area 1GB above start of text.
+     *
+     * Also, the length requires a pointer as the actual length is
+     * returned (rounded up to a page boundary).
+     */
+    {
+	unsigned len = SCOREBOARD_SIZE;
+	
+	m = mmap((caddr_t)0xC0000000, &len,
+		PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, NOFD, 0);
+    }
+#else
     m = mmap((caddr_t)0, SCOREBOARD_SIZE,
 	     PROT_READ | PROT_WRITE, MAP_ANON | MAP_SHARED, -1, 0);
+#endif
     if (m == (caddr_t)-1)
     {
 	perror("mmap");
@@ -1757,8 +1776,16 @@ void child_main(int child_num_arg)
                 exit(0);
 
             errno = errsave;
-            if (srv < 0 && errno != EINTR)
+            if (srv < 0 && errno != EINTR) {
+#ifdef LINUX
+		if (errno == EFAULT) {
+		    log_unixerr("select", "(listen) fatal, exiting",
+				NULL, server_conf);
+		    exit(1);
+		}
+#endif
                 log_unixerr("select", "(listen)", NULL, server_conf);
+	    }
 
             if (srv <= 0)
                 continue;
@@ -1982,9 +2009,16 @@ static int make_sock(pool *pconf, const struct sockaddr_in *server)
         exit(1);
     }
 
+    /* Solaris (probably versions 2.4, 2.5, and 2.5.1 with various levels
+     * of tcp patches) has some really weird bugs where if you dup the
+     * socket now it breaks things across SIGHUP restarts.  It'll either
+     * be unable to bind, or it won't respond.
+     */
+#ifndef SOLARIS2
     s = ap_slack(s, AP_SLACK_HIGH);
 
     note_cleanups_for_fd(pconf, s); /* arrange to close on exec or restart */
+#endif
     
 #ifndef MPE
 /* MPE does not support SO_REUSEADDR and SO_KEEPALIVE */
@@ -2053,6 +2087,12 @@ static int make_sock(pool *pconf, const struct sockaddr_in *server)
     if (ntohs(server->sin_port) < 1024) GETUSERMODE();
 #endif
     listen(s, 512);
+
+#ifdef SOLARIS2
+    s = ap_slack(s, AP_SLACK_HIGH);
+
+    note_cleanups_for_fd(pconf, s); /* arrange to close on exec or restart */
+#endif
     return s;
 }
 
@@ -2278,6 +2318,8 @@ void standalone_main(int argc, char **argv)
 	}
 
 	/* we've been told to restart */
+        signal (SIGHUP, SIG_IGN);
+        signal (SIGUSR1, SIG_IGN);
 
 	if (one_process) {
 	    /* not worth thinking about */
@@ -2395,17 +2437,8 @@ main(int argc, char *argv[])
     setup_prelinked_modules();
 
     suexec_enabled = init_suexec();
+    /* NeoSoft fix - see below */
 
-    /* Don't bother with first pass if we already know we are standalone
-       which is a NeoSoft-customized default */
-    if (standalone) {
-        standalone_main(argc, argv);
-        exit(0);
-    }
-
-    server_conf = read_config (pconf, ptrans, server_confname);
-    init_modules (pconf, server_conf);
-    
     if(standalone) {
         clear_pool (pconf);	/* standalone_main rereads... */
         standalone_main(argc, argv);
@@ -2416,6 +2449,11 @@ main(int argc, char *argv[])
 	struct sockaddr sa_server, sa_client;
 	BUFF *cio;
       
+        /* NeoSoft fix - do this if it not standalone, saves time by
+                preventing httpd from repeating redundant work */
+	server_conf = read_config (pconf, ptrans, server_confname);
+	init_modules (pconf, server_conf);
+
 	open_logs(server_conf, pconf);
 	set_group_privs();
 	default_server_hostnames (server_conf);
@@ -2437,6 +2475,9 @@ main(int argc, char *argv[])
           GETUSERMODE();
       }
 #endif
+        if (ap_setjmp (jmpbuffer)) {
+	    exit (0);
+        }
 
 	c = sizeof(sa_client);
 	if ((getpeername(fileno(stdin), &sa_client, &c)) < 0)
