@@ -344,7 +344,7 @@ int directory_walk (request_rec *r)
         core_dir_config *core_dir =
 	  (core_dir_config *)get_module_config(per_dir_defaults, &core_module);
 	int overrides_here;
-        void *this_conf = NULL, *htaccess_conf = NULL;
+        void *this_conf, *htaccess_conf = NULL;
 	char *this_dir = make_dirstr (r->pool, test_filename, i);
 	int j;
       
@@ -373,6 +373,7 @@ int directory_walk (request_rec *r)
 	      (core_dir_config *)get_module_config(entry_config, &core_module);
 	    entry_dir = entry_core->d;
 	
+	    this_conf = NULL;
 	    if (entry_core->r) {
 		if (!regexec(entry_core->r, this_dir, 0, NULL,
 			     (j == num_sec) ? 0 : REG_NOTEOL)) {
@@ -708,22 +709,31 @@ request_rec *sub_req_lookup_file (const char *new_file, const request_rec *r)
 
 	rnew->per_dir_config = r->per_dir_config;
 
-	if ((res = check_symlinks (rnew->filename, allow_options (rnew)))) {
-	    log_reason ("Symbolic link not allowed", rnew->filename, rnew);
-	    rnew->status = res;
-	    return rnew;
-	}
-	/* do a file_walk, if it doesn't change the per_dir_config then
-	 * we know that we don't have to redo all the access checks */
-	if ((res = file_walk (rnew))) {
-	    rnew->status = res;
-	    return rnew;
-	}
-	if (rnew->per_dir_config == r->per_dir_config) {
-	    if ((res = find_types (rnew)) || (res = run_fixups (rnew))) {
+	/* no matter what, if it's a subdirectory, we need to re-run
+	 * directory_walk */
+	if (S_ISDIR (rnew->finfo.st_mode)) {
+	    res = directory_walk (rnew);
+	    if (!res) {
+		res = file_walk (rnew);
+	    }
+	} else {
+	    if ((res = check_symlinks (rnew->filename, allow_options (rnew)))) {
+		log_reason ("Symbolic link not allowed", rnew->filename, rnew);
 		rnew->status = res;
+		return rnew;
+	    }
+	    /* do a file_walk, if it doesn't change the per_dir_config then
+	     * we know that we don't have to redo all the access checks */
+	    if ((res = file_walk (rnew))) {
+		rnew->status = res;
+		return rnew;
+	    }
+	    if (rnew->per_dir_config == r->per_dir_config) {
+		if ((res = find_types (rnew)) || (res = run_fixups (rnew))) {
+		    rnew->status = res;
+		}
+		return rnew;
 	    }
-	    return rnew;
 	}
     } else {
 	/* XXX: this should be set properly like it is in the same-dir case
@@ -896,9 +906,12 @@ void process_request_internal (request_rec *r)
 	((r->proto_num == 1001) && !table_get(r->headers_in, "Host"))) {
         /* Client sent us a HTTP/1.1 or later request without telling
 	 * us the hostname, either with a full URL or a Host: header.
-	 * We therefore need to (as per the 1.1 spec) send an error
+	 * We therefore need to (as per the 1.1 spec) send an error.
+	 * As a special case, HTTP/1.1 mentions twice (S9, S14.23)
+	 * that a request MUST contain a Host: header, and the server
+	 * MUST respond with 400 if it doesn't.
 	 */
-        log_reason ("client sent HTTP/1.1 request without hostname",
+        log_reason ("client sent HTTP/1.1 request without hostname (see RFC2068 sections 9 and 14.23)",
 		    r->uri, r);
 	die (BAD_REQUEST, r);
 	return;
@@ -910,8 +923,10 @@ void process_request_internal (request_rec *r)
          * we handle it specially.
          */
         if (r->method_number == M_TRACE) {
-            send_http_trace(r);
-            finalize_request_protocol(r);
+            if ((access_status = send_http_trace(r)))
+	        die(access_status, r);
+            else
+                finalize_request_protocol(r);
             return;
         }
 
@@ -935,6 +950,10 @@ void process_request_internal (request_rec *r)
 	return;
     }
 
+    if (r->proto_num > 1000 && table_get (r->subprocess_env, "downgrade-1.0")) {
+	r->proto_num = 1000;
+    }
+
     /* NB: directory_walk() clears the per_dir_config, so we don't inherit from
        location_walk() above */
 
