@@ -256,14 +256,17 @@
  * Example: "Apache/1.1.0 MrWidget/0.1-alpha" 
  */
 
-#define SERVER_BASEVERSION "Apache/1.2.1" /* SEE COMMENTS ABOVE */
+#define SERVER_BASEVERSION "NeoWebScript/2.3 Apache/1.2.5" /* SEE COMMENTS ABOVE */
 #ifdef SERVER_SUBVERSION
 #define SERVER_VERSION	SERVER_BASEVERSION " " SERVER_SUBVERSION
 #else
 #define SERVER_VERSION	SERVER_BASEVERSION
 #endif
 
-#define APACHE_RELEASE_DATE 19970705
+/* Numeric release version identifier: major minor bugfix betaseq
+ * Always increases along the same track as the source branch.
+ */
+#define APACHE_RELEASE 1020501
 
 #define SERVER_PROTOCOL "HTTP/1.1"
 #define SERVER_SUPPORT "http://www.apache.org/"
