@@ -1,10 +1,10 @@
 #
-# NeoScript - Server Side Programming based on Safe Tcl
+# NeoWebScript - Server Side Programming based on Safe Tcl
 #
 # Copyright (C) NeoSoft, All Rights Reserved.  See NeoWebScript LICENSE
 # files with this distribution for conditions on use and redistribution.
 #
-# $Id: init.tcl,v 1.31 1997/07/24 21:10:54 karl Exp $
+# $Id: init.tcl,v 1.50 1998/01/05 21:11:08 tsunami Exp $
 #
 
 #
@@ -119,48 +119,65 @@ safe_and_trusted_alias backlink {{linktext ""} {nolinktext ""}} {
 # PROPERTY LIST STORAGE AND RETRIEVAL
 #
 
-SAFE_proc_and_alias dbfetch {safeInterp database id arrayName {singleVar ""}} {
-    global parallelDir errorCode errorInfo
+SAFE_proc_and_alias dbfetch {safeInterp database id arrayName args} {
+    global errorCode
 
     setup_data_access
-    db_name_check $database db dbFileName
 
-    if {[catch {set db [db open $dbFileName hash rwl 0664]} result] == 1} {
+    set ssearch [lsearch -exact $args "-singleVar"]
+    if {$ssearch > -1} {	
+	set singleVar "-singleVar"
+	set args [lreplace $args $ssearch $ssearch]
+    } else {
+	set singleVar {}
+    }
+
+    if {[cindex $args 0] == "-"} {import_keyvalue_pairs data $args}
+    if ![info exists data(project)] {set data(project) {}}
+    db_name_check $database db dbFileName $data(project)
+
+    if {[catch {db open $dbFileName hash rwl 0664} db] == 1} {
 	if {[lrange $errorCode 0 1] == "POSIX ENOENT"} {
 	    return 0
 	} else {
 	    error "$dbFileName: $errorCode" $errorInfo
 	}
     }
+
     db get $db $id list
     db close $db
 
     set haveData [expr ![lempty $list]]
 
-	if {$singleVar == "-singleVar"} {
-		$safeInterp eval set $arrayName $list
-	} else {
-		$safeInterp eval array set $arrayName [list $list]
-	}
+    if {$singleVar == "-singleVar"} {
+	$safeInterp eval set $arrayName [list $list]
+    } else {
+	$safeInterp eval array set $arrayName [list $list]
+    }
 
     return $haveData
 }
 
-SAFE_proc_and_alias dbstore {safeInterp database id arrayName {singleVar ""}} {
-    global parallelDir errorCode errorInfo
+SAFE_proc_and_alias dbstore {safeInterp database id arrayName args} {
+    global errorCode
 
     setup_data_access
-    db_name_check $database db dbFileName
 
-	if {$singleVar == "-singleVar"} {
-		set list $arrayName
-	} else {
-		set list [$safeInterp eval array get $arrayName]
-	}
+    set ssearch [lsearch -exact $args "-singleVar"]
+    if {$ssearch > -1} {	
+	set list $arrayName
+	set args [lreplace $args $ssearch $ssearch]
+    } else {
+	set list [$safeInterp eval array get $arrayName]
+    }
+
+    if {[cindex $args 0] == "-"} {import_keyvalue_pairs data $args}
+    if ![info exists data(project)] {set data(project) {}}
+    db_name_check $database db dbFileName $data(project)
 
     if {[catch {set db [db open $dbFileName hash rwL 0664]}] == 1} {
 	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
-	    error "$dbFileName: $errorCode" $errorInfo
+	    return -code error "$dbFileName: $errorCode"
 	} else {
 	    create_user_dir db
 	    set db [db open $dbFileName hash ctL 0664]
@@ -240,12 +257,9 @@ SAFE_proc_and_alias load_response {safeInterp {responseVar "response"} {multiple
 SAFE_proc_and_alias incr_page_counter {safeInterp {start 0}} {
     global webenv
 
-    if {[catch {dbfetch pagecounters $webenv(DOCUMENT_URI) accessInfo}] == 1} {
-	set accessInfo(counter) $start
-    } elseif {$start > $accessInfo(counter)} {
-	set accessInfo(counter) $start
-	}
-    if ![info exists accessInfo(counter)] {
+    if {![dbfetch pagecounters $webenv(DOCUMENT_URI) accessInfo] \
+      || ![info exists accessInfo(counter)] \
+      || $start > $accessInfo(counter)} {
 	set accessInfo(counter) $start
     }
     set counter [incr accessInfo(counter)]
@@ -257,30 +271,30 @@ SAFE_proc_and_alias incr_page_counter {safeInterp {start 0}} {
 # Pick an element in a list at random and return it
 # Now uses keyvalue pairs and will allow a text file as its list
 #
-proc random_pick_html {args} {
+SAFE_proc_and_alias random_pick_html {safeInterp args} {
     if {[cindex $args 0] == "-"} {
-		if {[lindex [lindex $args 0] 0] == "-file"} {
-			import_keyvalue_pairs data $args
-		}
+	import_keyvalue_pairs data $args
     }
 
     if [info exists data(file)] {
-		if {[string match */* $data(file)] || [string match .. $data(file)]} {
-			error "You may not open a file outside of the current working directory."
-			return
-		}
+	set nFilename [SAFE_filename $data(file)]
+	if [lempty $nFilename] {
+	    return -code error "$data(file): Invalid pathname requested"
+	}
 
-        if [file exists [pwd]/$data(file)] {
-            set fp [open [pwd]/$data(file) r]
+        if [file exists $nFilename] {
+            set fp [open $nFilename r]
             set list ""
             while {[gets $fp line] != -1} {
                 lappend list $line
             }
             close $fp
         } else {
-            error "$data(file): file does not exist"
+            return -code error "$data(file): file does not exist"
         }
-    } else {set list [lindex $args 0]}
+    } else {
+	set list [lindex $args 0]
+    }
 
     return [lindex $list [expr [clock seconds] % [llength $list]]]
 }
@@ -288,7 +302,6 @@ proc random_pick_html {args} {
 #
 # Log a message to the log directory we keep for the users.
 #
-#
 SAFE_proc_and_alias log_message {safeInterp logfile string} {
     global parallelDir errorCode errorInfo webenv
 
@@ -311,11 +324,13 @@ SAFE_proc_and_alias log_message {safeInterp logfile string} {
 #
 # Access a data file with all the file stuff present in safe interpreters.
 #
-SAFE_proc_and_alias access_data_file {safeInterp datafile} {
+SAFE_proc_and_alias access_data_file {safeInterp datafile args} {
     global parallelDir errorCode
     setup_data_access
 
-    db_name_check $datafile data dataFileName
+    if {[cindex $args 0] == "-"} {import_keyvalue_pairs data $args}
+    if ![info exists data(project)] {set data(project) {}}
+    db_name_check $datafile data dataFileName $data(project)
 
     if {[catch {set fp [open $dataFileName "CREAT RDWR" 0660]}] == 1} {
 	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
@@ -329,27 +344,34 @@ SAFE_proc_and_alias access_data_file {safeInterp datafile} {
     return $fp
 }
 
-SAFE_proc_and_alias delete_data_file {safeInterp datafile} {
+SAFE_proc_and_alias delete_data_file {safeInterp datafile} { 
     global parallelDir errorCode
-    setup_data_access
-
+    setup_data_access 
+    
     db_name_check $datafile data dataFileName
-
+    
     if {[catch {unlink $dataFileName} result] == 1} {
-	if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
-	    error "$result"
-	}
+        if {[lrange $errorCode 0 1] != "POSIX ENOENT"} {
+            error "$result"
+        }
     }
     return
 }
 
 #
-# Send mail
-#   
+# Send mail configuration: if no NeoWebServerConf SendMail
+#
+if {![info exists $NeoWebServerConf(SendMail)]} {
+    if {[file executable /usr/sbin/sendmail]} {
+        set NeoWebServerConf(SendMail) /usr/sbin/sendmail
+    } elseif {[file executable /usr/lib/sendmail]} {
+        set NeoWebServerConf(SendMail) /usr/lib/sendmail
+    }
+}
 
 SAFE_proc_and_alias open_outbound_mail {safeInterp args} {
     
-    global webpageOwnerName webenv
+    global webpageOwnerName webenv NeoWebServerConf
  
     if {[cindex $args 0] != "-"} {
 	lassign $args data(subject) data(to) data(from)
@@ -374,11 +396,11 @@ SAFE_proc_and_alias open_outbound_mail {safeInterp args} {
     regsub -all " " $data(from) "" data(from)
 
 
-    set fp [open "|/usr/lib/sendmail -t -f $data(from)" w]
+    set fp [open "|$NeoWebServerConf(SendMail) -t -f $data(from)" w]
     puts $fp "From: $data(from)"
     puts $fp "To: $data(to)"
     puts $fp "Subject: $data(subject)"
-    foreach header [lmatch [array names data] {[A-Z*]}] {
+    foreach header [lmatch [array names data] {[A-Z]*}] {
 	if [info exists data($header)] {
 	    puts $fp "$header: $data($header)"
 	}
@@ -462,9 +484,19 @@ SAFE_proc_and_alias open_post_news {safeInterp args} {
 }
 
 
-SAFE_proc_and_alias directory_listing {safeInterp {pattern *}} {
+#
+# Disk usage configuration: if no NeoWebServerConf Du
+#
+if {![info exists $NeoWebServerConf(Du)]} {
+    if {[file executable /usr/bin/du]} {
+        set NeoWebServerConf(Du) /usr/bin/du
+    }
+}
 
-    set duFP [open "|du -s" r]
+SAFE_proc_and_alias directory_listing {safeInterp {pattern *}} {
+    global NeoWebServerConf
+
+    set duFP [open "|$NeoWebServerConf(Du) -sk" r]
     gets $duFP line
     close $duFP
 
@@ -528,21 +560,20 @@ safe_and_trusted_alias remote_hostname {{ip 0}} {
     global webenv
 
     if {$ip != 0} {
-       if {[catch {host_info official_name $ip} result] == 1} {
+	if {[catch {host_info official_name $ip} result] == 1} {
 	   return $ip
-       }
+	}
     } else {
-       if {$webenv(REMOTE_HOST) != $webenv(REMOTE_ADDR)} {
+	if {$webenv(REMOTE_HOST) != $webenv(REMOTE_ADDR)} {
 	   return $webenv(REMOTE_HOST)
-       }
-       if {[catch {host_info official_name $webenv(REMOTE_ADDR)} result] == 1} {
+	}
+	if {[catch {host_info official_name $webenv(REMOTE_ADDR)} result] == 1} {
 	   return $webenv(REMOTE_ADDR)
-       }
+	}
     }
     return $result
 }
 
-
 #
 # destroy the safe interpreter if it already exists, then
 # create a new one and install our services (exported
@@ -565,30 +596,16 @@ proc setup_safe_interpreter {} {
 
     load {} Tclx $safeInterp
 
-    $safeInterp alias dbfetch SAFE_dbfetch $safeInterp
-    $safeInterp alias dbstore SAFE_dbstore $safeInterp
-    $safeInterp alias dbkeys dbkeys
-    $safeInterp alias dbexists dbexists
-    $safeInterp alias dbdelkey dbdelkey
-    $safeInterp alias dbfiles dbfiles
-
     $safeInterp alias log_message SAFE_log_message $safeInterp
 
     $safeInterp alias access_data_file SAFE_access_data_file $safeInterp
     $safeInterp alias delete_data_file SAFE_delete_data_file $safeInterp
 
-    $safeInterp alias list_data_files list_data_files
-	$safeInterp alias data_file_exists data_file_exists
-
-    $safeInterp alias filerm filerm
-    $safeInterp alias filemv filemv
-
     $safeInterp alias load_cookies SAFE_load_cookies $safeInterp
     $safeInterp alias load_response SAFE_load_response $safeInterp
 
     $safeInterp alias incr_page_counter SAFE_incr_page_counter $safeInterp
 
-    $safeInterp alias random_pick_html random_pick_html
     $safeInterp alias abort_page abort_page
     $safeInterp alias flush_page flush_page
 
@@ -601,9 +618,11 @@ proc setup_safe_interpreter {} {
 
     $safeInterp alias unquote_string www_unescape_string
     $safeInterp alias quote_string www_escape_string
+    $safeInterp alias escape_sgml_chars www_escape_sgml_chars
 
     $safeInterp alias estimate_hits_per_hour estimate_hits_per_hour
     $safeInterp alias remote_hostname remote_hostname
+    $safeInterp alias gm_timestr_822 gm_timestr_822
 
     extend_slave $safeInterp
 
@@ -691,7 +710,7 @@ proc handle_server_side_eval {safeInterp ssiCode} {
     global debugging errorInfo
 
     if {[catch {$safeInterp eval $ssiCode} result] == 1} {
-	html "\{NEOSCRIPT ERROR: $result\}"
+	html "\{NEOWEBSCRIPT ERROR: $result\}"
 	if {$debugging} {
 	    html "\n<pre>$errorInfo</pre>"
 	}
@@ -706,7 +725,7 @@ proc handle_server_side_return {safeInterp ssiCode} {
     global debugging errorInfo
 
     if {[catch {$safeInterp eval $ssiCode} result] == 1} {
-	html "\{NEOSCRIPT ERROR: $result\}"
+	html "\{NEOWEBSCRIPT ERROR: $result\}"
 	if {$debugging} {
 	    html "\n<pre>$errorInfo</pre>"
 	}
@@ -725,7 +744,7 @@ proc handle_server_side_variable {safeInterp ssiVar} {
     global debugging
 
     if {[catch {$safeInterp eval set $ssiVar} result] == 1} {
-	html "\{NEOSCRIPT ERROR: $result\}"
+	html "\{NEOWEBSCRIPT ERROR: $result\}"
 	if {$debugging} {
 	    html "\n<pre>$errorInfo</pre>"
 	}
@@ -744,7 +763,7 @@ proc handle_server_side_expr {safeInterp ssiExpr} {
     global debugging
 
     if {[catch {$safeInterp eval expr $ssiExpr} result] == 1} {
-	html "\{NEOSCRIPT ERROR: $result\}"
+	html "\{NEOWEBSCRIPT ERROR: $result\}"
 	if {$debugging} {
 	    html "\n<pre>$errorInfo</pre>"
 	}
@@ -803,16 +822,16 @@ proc setup_slave_interp_unknown {safeInterp} {
 }
 
 proc SAFE_autoload {safeInterp cmd} {
-	global safe_proc_cache safe_alias_cache
-	if [info exists safe_proc_cache($cmd)] {
-		$safeInterp eval $safe_proc_cache($cmd)
-		return 1
-	}
-	if [info exists safe_alias_cache($cmd)] {
-		eval $safeInterp alias $cmd [subst $safe_alias_cache($cmd)]
-		return 1
-	}
-	return 0
+    global safe_proc_cache safe_alias_cache
+    if [info exists safe_proc_cache($cmd)] {
+	$safeInterp eval $safe_proc_cache($cmd)
+	return 1
+    }
+    if [info exists safe_alias_cache($cmd)] {
+	eval $safeInterp alias $cmd [subst $safe_alias_cache($cmd)]
+	return 1
+    }
+    return 0
 }
 
 proc SAFE_pkgUnknown {safeInterp name {version ""}} {
@@ -842,10 +861,10 @@ proc safe_pkgIfNeeded {name body} {
 }
 
 proc read_file {filename} {
-	set fp [open $filename]
-	set data [read $fp]
-	close $fp
-	return $data
+    set fp [open $filename]
+    set data [read $fp]
+    close $fp
+    return $data
 }
 
 if {[info commands import] == ""} {
@@ -861,38 +880,45 @@ proc SAFE_filename {filename} {
 	set filename [file join [file dirname $webenv(SCRIPT_FILENAME)] $filename]
     }
     set filename [www_simplify_pathname $filename]
-    if [lsearch -exact $SafePaths [file dirname $filename]]<0 {
-	return
+    if {[lsearch -exact $SafePaths [file dirname $filename]] < 0} {
+        return 
     }
+    #if ![isfile_safe $filename] return
+
     return $filename
 }
 
 proc SAFE_open {safeInterp filename args} {
-    set nfilename [SAFE_filename $filename]
-    if [lempty $nfilename] {
-	error "$filename: Invalid pathname requested"
+    set nFilename [SAFE_filename $filename]
+
+    if [lempty $nFilename] {
+	return -code error "$filename: Invalid pathname requested"
     }
-    set channel [open $nfilename]
+    set channel [open $nFilename]
     interp transfer {} $channel $safeInterp
     return $channel
 }
 
 proc SAFE_source {safeInterp filename} {
-    set nfilename [SAFE_filename $filename]
-    if [lempty $nfilename] {
-	error "$filename: Invalid pathname requested"
+    set nFilename [SAFE_filename $filename]
+
+    if [lempty $nFilename] {
+	return -code error "$filename: Invalid pathname requested"
     }
-    $safeInterp eval [read_file $nfilename]
+    $safeInterp eval [read_file $filename]
 }
 
 proc isfile_safe {fileName} {
-	if {[cindex $fileName 0] == "/" || [string match *..* $fileName]} {return 0}
-	return 1
+    global SafePaths
+    if {[cindex $fileName 0] == "/" || [string match *..* $fileName] || [string match */.* $fileName] || [string match *~* $fileName]} {
+	return 0
+    }
+    return 1
 }
-	
+
 proc SAFE_glob {safeInterp args} {
     foreach pattern $args {
-		if ![isfile_safe $pattern] {
+	if ![isfile_safe $pattern] {
             return -code error "Illegal filename: $pattern"
         }
     }
@@ -900,7 +926,9 @@ proc SAFE_glob {safeInterp args} {
 }
 
 proc SAFE_read_file {safeInterp filename} {
-	if ![isfile_safe $filename] {
+	set nFilename [SAFE_filename $filename]
+
+	if [lempty $nFilename] {
         return -code error "Illegal filename: $filename"
     }
     return [read_file $filename]
